unit kolxsocket;

interface

uses
  winsock, kolxtimer, kol;

type
  PBaseSocket = ^TBaseSocket;
  PReceive = ^TReceive;
  PClient = ^TClient;
  PServer = ^TServer;
  PAccept = ^TAccept;
  PServerClient = ^TServerClient;

  TKOLClientSocket = PClient;
  TKOLServerSocket = PServer;

  TOnReceive = procedure(Sender: PBaseSocket; var Buf: array of char; const Count: Integer) of object;
  TOnDisconnect = procedure(Sender: PBaseSocket) of object;
  TOnResolve = procedure(Sender: PClient; const IP: String) of object;
  TOnConnect = procedure(Sender: PClient) of object;
  TOnAccept = procedure(Sender: PServer; const IP: String;
    const Port: SmallInt; var ServerClient: PServerClient) of object;
  TOnError = procedure(Sender: PObj; const Error:integer) of object;

  TBaseSocket = object(TObj)
  private
    Sock: TSocket;
    fOnline: Boolean;
    Receive: PReceive;
  protected
    destructor Destroy;virtual;
  public
    Tag: Integer;
    Index: Integer;
    OnReceive: TOnReceive;
    OnDisconnect: TOnDisconnect;
    OnError: TOnError;
    procedure Disconnect;
    procedure Send(var Buf; Count: Integer);
    property Online: Boolean read fOnline;
    function ErrorTest(const rslt: integer): boolean;
  end;

  TReceive = object(TObj)
  public
    Base: PBaseSocket;
    Buf: array [0..4095] of Char;
    Thread: PThread;
    function Execute(sender: pthread): integer;
  end;

  TClient = object(TBaseSocket)
  private
    ConnectThread: PThread;
    function ConnectExecute(sender: pthread): integer;
  public
    Host: String;
    Port: SmallInt;
    OnResolve: TOnResolve;
    OnConnect: TOnConnect;
    procedure Connect;
  end;

  TServer = object(TObj)
  private
    Sock: TSocket;
    Timer: PXTimer;
    Accept: PAccept;
    Connections: PList;
    ListenThread: PThread;
    function GCount: Integer;
    function GConnection(Index: Integer): PServerClient;
    procedure OTimer(Sender: PXTimer);
    function ListenExecute(sender: pthread):integer;
  protected
    destructor Destroy;virtual;  
  public
    Port: SmallInt;
    Tag: Integer;
    OnAccept: TOnAccept;
    OnError: TOnError;
    OnClientError: TOnError;
    OnClientDisconnect: TOnDisconnect;
    OnClientReceive: TOnReceive;
    function ErrorTest(const rslt: integer): boolean;
    procedure Listen;
    procedure Disconnect;
    property Count: Integer read GCount;
    property Connection[Index: Integer]: PServerClient read GConnection;
  end;

  TAccept = object(TObj)
  public
    Thread: PThread;
    Server: PServer;
    function Execute(sender: pthread):integer;
  end;

  TServerClient = object(TBaseSocket)
  public
    IP: String;
    Port: SmallInt;
    Server: PServer;
  end;

function NewReceive: PReceive;
function NewAccept: PAccept;
function NewServer: PServer;
function NewServerClient: PServerClient;
function NewClient: PClient;
function Err2Str(const id: integer): string;
function GetHostByName(const host: pchar): string;

implementation

{ TBaseSocket }

destructor TBaseSocket.Destroy;
begin
  disconnect;
  inherited;
end;

procedure TBaseSocket.Disconnect;
begin
  if assigned(receive) then
  begin
    closesocket(sock);
    receive:=nil;
  end;
end;

function TBaseSocket.ErrorTest(const rslt: integer): boolean;
var
  err:integer;
begin
  result:=rslt=SOCKET_ERROR;
  if result then
  begin
    err:=wsagetlasterror;
    if assigned(onerror) then onerror(@self,err) else
      showmessage('Socket error '+err2str(err)+' on socket '+int2str(sock));
  end;
end;

procedure TBaseSocket.Send;
begin
  errortest(winsock.send(sock,buf,count,0));
end;

{ TReceive }

function TReceive.Execute(sender: pthread): integer;
var
  i:integer;
begin
  with base^ do
  begin
    fonline:=true;
    repeat
      i:=recv(sock,buf,sizeof(buf),0);
      if (i=SOCKET_ERROR) or (i=0) then break;
      if assigned(onreceive) then onreceive(base,buf,i);
    until false;
    if assigned(ondisconnect) then ondisconnect(base);
    fonline:=false;
  end;
  result:=0;
end;

{ TClient }

function TClient.ConnectExecute(sender: pthread): integer;
var
  adr:tsockaddr;
  ph:phostent;
begin
  disconnect;
  sock:=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
  if not errortest(sock) then
  begin
    with adr do
    begin
      sin_family:=AF_INET;
      sin_port:=htons(port);
      integer(sin_addr):=inet_addr(pchar(host));
      if integer(sin_addr)=SOCKET_ERROR then
      begin
        ph:=winsock.gethostbyname(pchar(host));
        if ph=nil then showmessage('gethostbyname() error');
        move(ph.h_addr^^,sin_addr,ph.h_length);
        if assigned(onresolve) then onresolve(@self,inet_ntoa(adr.sin_addr));
      end;
    end;
    if not errortest(winsock.connect(sock,adr,sizeof(adr))) then
    begin
      receive:=newreceive;
      receive.base:=@self;
      receive.thread.resume;
      if assigned(onconnect) then onconnect(@self);
    end;
  end;
  result:=0;
end;

procedure TClient.Connect;
begin
  connectthread:=newthreadautofree(connectexecute);
  connectthread.resume;
end;

{ TServer }

procedure TServer.OTimer;
var
  i,j:integer;
begin
  for i:=pred(count) downto 0 do
    if not connection[i].online then
    begin
      for j:=pred(count) downto i do dec(connection[i].index);
      connection[i].free;
      connections.delete(i);
    end;
end;

function TServer.GCount;
begin
  result:=connections.count;
end;

function TServer.GConnection;
begin
  result:=pserverclient(connections.items[index]);
end;

destructor TServer.Destroy;
begin
  disconnect;
  while count<>0 do;
  timer.free;
  connections.Free;
  inherited;
end;

function TServer.ListenExecute(sender: pthread): integer;
var
  adr:tsockaddr;
begin
  sock:=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
  if not errortest(sock) then
  begin
    with adr do
    begin
      sin_family:=AF_INET;
      sin_port:=htons(port);
      integer(sin_addr):=INADDR_ANY;
    end;
    if not errortest(bind(sock,adr,sizeof(adr))) and not errortest(winsock.listen(sock,64)) then
    begin
      accept:=newaccept;
      accept.server:=@self;
      accept.thread.resume;
    end;
  end;
  result:=0;
end;

procedure TServer.Listen;
begin
  disconnect;
  listenthread:=newthreadautofree(listenexecute);
  listenthread.resume;
end;

function TServer.ErrorTest(const rslt: integer): boolean;
var
  err:integer;
begin
  result:=rslt=SOCKET_ERROR;
  if result then
  begin
    err:=wsagetlasterror;
    if assigned(onerror) then onerror(@self,err) else
      showmessage('Socket error '+err2str(err)+' on socket '+int2str(sock));
  end;
end;

procedure TServer.Disconnect;
var
  i:integer;
begin
  if assigned(accept) then
  begin
    closesocket(sock);
    accept:=nil;
  end;
  for i:=0 to pred(count) do connection[i].disconnect;
end;

function TAccept.Execute(sender: pthread): integer;
var
  sock:tsocket;
  adr:tsockaddr;
  pi:integer;
  serverclient:pserverclient;
begin
  pi:=sizeof(adr);
  repeat
    sock:=accept(server.sock, @adr, @pi);
    if sock=SOCKET_ERROR then Break;
    serverclient:=nil;
    if assigned(server.onaccept) then
      server.onaccept(server,inet_ntoa(adr.sin_addr),htons(adr.sin_port),serverclient);
    if assigned(serverclient) then
    begin
      serverclient.server:=server;
      serverclient.sock:=sock;
      with serverclient^ do
      begin
        onerror:=server.onclienterror;
        onreceive:=server.onclientreceive;
        ondisconnect:=server.onclientdisconnect;
        ip:=inet_ntoa(adr.sin_addr);
        port:=htons(adr.sin_port);
        receive:=newreceive;
        receive.base:=serverclient;
        receive.thread.resume;
      end;
      serverclient.index:=server.connections.count;
      server.connections.add(serverclient);
    end else closesocket(sock);
  until false;
  result:=0;
end;

var
  wsa: WSAData;

function NewReceive: PReceive;
begin
  new(result,create);             
  result.thread:=newthreadautofree(nil);
  result.thread.onexecute:=result.execute;
end;

function NewAccept: PAccept;
begin
  new(result,create);
  result.thread:=newthreadautofree(nil);
  result.thread.onexecute:=result.execute;
end;

function NewServer: PServer;
begin
  new(result,create);
  result.timer:=newxtimer;
  result.timer.ontimer:=result.otimer;
  result.connections:=newlist;
end;

function NewServerClient: PServerClient;
begin
  new(result,create);
end;

function NewClient: PClient;
begin
  new(result,create);
end;

function Err2Str(const id: integer): string;
begin
  case id of
  WSAEINTR:result:='WSAEINTR';
  WSAEBADF:result:='WSAEBADF';
  WSAEACCES:result:='WSAEACCES';
  WSAEFAULT:result:='WSAEFAULT';
  WSAEINVAL:result:='WSAEINVAL';
  WSAEMFILE:result:='WSAEMFILE';
  WSAEWOULDBLOCK:result:='WSAEWOULDBLOCK';
  WSAEINPROGRESS:result:='WSAEINPROGRESS';
  WSAEALREADY:result:='WSAEALREADY';
  WSAENOTSOCK:result:='WSAENOTSOCK';
  WSAEDESTADDRREQ:result:='WSAEDESTADDRREQ';
  WSAEMSGSIZE:result:='WSAEMSGSIZE';
  WSAEPROTOTYPE:result:='WSAEPROTOTYPE';
  WSAENOPROTOOPT:result:='WSAENOPROTOOPT';
  WSAEPROTONOSUPPORT:result:='WSAEPROTONOSUPPORT';
  WSAESOCKTNOSUPPORT:result:='WSAESOCKTNOSUPPORT';
  WSAEOPNOTSUPP:result:='WSAEOPNOTSUPP';
  WSAEPFNOSUPPORT:result:='WSAEPFNOSUPPORT';
  WSAEAFNOSUPPORT:result:='WSAEAFNOSUPPORT';
  WSAEADDRINUSE:result:='WSAEADDRINUSE';
  WSAEADDRNOTAVAIL:result:='WSAEADDRNOTAVAIL';
  WSAENETDOWN:result:='WSAENETDOWN';
  WSAENETUNREACH:result:='WSAENETUNREACH';
  WSAENETRESET:result:='WSAENETRESET';
  WSAECONNABORTED:result:='WSAECONNABORTED';
  WSAECONNRESET:result:='WSAECONNRESET';
  WSAENOBUFS:result:='WSAENOBUFS';
  WSAEISCONN:result:='WSAEISCONN';
  WSAENOTCONN:result:='WSAENOTCONN';
  WSAESHUTDOWN:result:='WSAESHUTDOWN';
  WSAETOOMANYREFS:result:='WSAETOOMANYREFS';
  WSAETIMEDOUT:result:='WSAETIMEDOUT';
  WSAECONNREFUSED:result:='WSAECONNREFUSED';
  WSAELOOP:result:='WSAELOOP';
  WSAENAMETOOLONG:result:='WSAENAMETOOLONG';
  WSAEHOSTDOWN:result:='WSAEHOSTDOWN';
  WSAEHOSTUNREACH:result:='WSAEHOSTUNREACH';
  WSAENOTEMPTY:result:='WSAENOTEMPTY';
  WSAEPROCLIM:result:='WSAEPROCLIM';
  WSAEUSERS:result:='WSAEUSERS';
  WSAEDQUOT:result:='WSAEDQUOT';
  WSAESTALE:result:='WSAESTALE';
  WSAEREMOTE:result:='WSAEREMOTE';
  WSASYSNOTREADY:result:='WSASYSNOTREADY';
  WSAVERNOTSUPPORTED:result:='WSAVERNOTSUPPORTED';
  WSANOTINITIALISED:result:='WSANOTINITIALISED';
  WSAHOST_NOT_FOUND:result:='WSAHOST_NOT_FOUND';
  WSATRY_AGAIN:result:='WSATRY_AGAIN';
  WSANO_RECOVERY:result:='WSANO_RECOVERY';
  WSANO_DATA:result:='WSANO_DATA';
  else result:='WSAEUNKNOWN';
  end;
end;

function GetHostByName(const host: pchar): string;
var
  ph:phostent;
  sa:in_addr;
begin
  ph:=winsock.gethostbyname(pchar(host));
  move(ph.h_addr^^,sa,ph.h_length);
  result:=inet_ntoa(sa);
end;

initialization
  if wsastartup($0101,wsa)<>0 then showmessage('WSAStartup() error.');

finalization
  wsacleanup;
end.
