unit mckXSocket;

interface

uses
  Windows, Classes, Messages, Winsock, Forms, SysUtils,
  KOLXSocket, mirror;

type
  TKOLClientSocket = class(TKOLObj)
  private
    FPort: smallint;
    FHost: string;
    FOnConnect: TOnConnect;
    FOnDisconnect: TOnDisconnect;
    FOnError: TOnError;
    FOnReceive: TOnReceive;
    FOnResolve: TOnResolve;
    procedure SetHost(const Value: string);
    procedure SetOnConnect(const Value: TOnConnect);
    procedure SetOnDisconnect(const Value: TOnDisconnect);
    procedure SetOnError(const Value: TOnError);
    procedure SetOnReceive(const Value: TOnReceive);
    procedure SetOnResolve(const Value: TOnResolve);
    procedure SetPort(const Value: smallint);
  protected
    function  AdditionalUnits: string; override;
    procedure SetupFirst( SL: TStringList; const AName, AParent, Prefix: String ); override;
    procedure SetupLast( SL: TStringList; const AName, AParent, Prefix: String ); override;
    procedure AssignEvents( SL: TStringList; const AName: String ); override;
  public
  published
    property Host:string read FHost write SetHost;
    property Port:smallint read FPort write SetPort;
    property OnDisconnect:TOnDisconnect read FOnDisconnect write SetOnDisconnect;
    property OnError:TOnError read FOnError write SetOnError;
    property OnReceive:TOnReceive read FOnReceive write SetOnReceive;
    property OnResolve:TOnResolve read FOnResolve write SetOnResolve;
    property OnConnect:TOnConnect read FOnConnect write SetOnConnect;
  end;                 

  TKOLServerSocket = class(TKOLObj)
  private
    FPort: smallint;
    FOnClientError: TOnError;
    FOnAccept: TOnAccept;
    FOnError: TOnError;
    FOnClientReceive: TOnReceive;
    FOnClientDisconnect: TOnDisconnect;
    procedure SetOnAccept(const Value: TOnAccept);
    procedure SetOnError(const Value: TOnError);
    procedure SetPort(const Value: smallint);
    procedure SetOnClientError(const Value: TOnError);
    procedure SetOnClientReceive(const Value: TOnReceive);
    procedure SetOnClientDisconnect(const Value: TOnDisconnect);
  protected
    function  AdditionalUnits: string; override;
    procedure SetupFirst( SL: TStringList; const AName, AParent, Prefix: String ); override;
    procedure SetupLast( SL: TStringList; const AName, AParent, Prefix: String ); override;
    procedure AssignEvents( SL: TStringList; const AName: String ); override;
  public
  published
    property Port:smallint read FPort write SetPort;
    property OnAccept:TOnAccept read FOnAccept write SetOnAccept;
    property OnError:TOnError read FOnError write SetOnError;
    property OnClientError:TOnError read FOnClientError write SetOnClientError;
    property OnClientReceive:TOnReceive read FOnClientReceive write SetOnClientReceive;
    property OnClientDisconnect:TOnDisconnect read FOnClientDisconnect write SetOnClientDisconnect;
  end;

  procedure Register;

implementation

{$R *.dcr}

procedure Register;
begin
  RegisterComponents('KOL', [TKOLClientSocket,TKOLServerSocket]);
end;

{ TKOLClientSocket }

function TKOLClientSocket.AdditionalUnits;
begin
  result:=', KOLXSocket';
end;

procedure TKOLClientSocket.SetupFirst(SL: TStringList; const AName, AParent, Prefix: String);
begin
  sl.add(prefix+aname+':=newclient;');
  sl.add(prefix+aname+'.port:='+inttostr(fport)+';');
  sl.add(prefix+aname+'.host:='#39+fhost+#39';');
end;

procedure TKOLClientSocket.SetupLast(SL: TStringList; const AName, AParent, Prefix: String);
begin
   //
end;

procedure TKOLClientSocket.AssignEvents(SL: TStringList; const AName: String);
begin
  inherited;
  doassignevents(sl,aname,
    ['OnConnect','OnDisconnect','OnError','OnReceive','OnResolve'],
    [@OnConnect,@OnDisconnect,@OnError,@OnReceive,@OnResolve]);
end;

procedure TKOLClientSocket.SetHost(const Value: string);
begin
  FHost := Value;
  change;
end;

procedure TKOLClientSocket.SetOnConnect(const Value: TOnConnect);
begin
  FOnConnect := Value;
  change;
end;

procedure TKOLClientSocket.SetOnDisconnect(const Value: TOnDisconnect);
begin
  FOnDisconnect := Value;
  change;
end;

procedure TKOLClientSocket.SetOnError(const Value: TOnError);
begin
  FOnError := Value;
  change;
end;

procedure TKOLClientSocket.SetOnReceive(const Value: TOnReceive);
begin
  FOnReceive := Value;
  change;
end;

procedure TKOLClientSocket.SetOnResolve(const Value: TOnResolve);
begin
  FOnResolve := Value;
  change;
end;

procedure TKOLClientSocket.SetPort(const Value: smallint);
begin
  FPort := Value;
  change;
end;

{ TKOLServerSocket }

function TKOLServerSocket.AdditionalUnits: string;
begin
  result:=', KOLXSocket';
end;

procedure TKOLServerSocket.AssignEvents(SL: TStringList;
  const AName: String);
begin
  inherited;
  doassignevents(sl,aname,
    ['OnAccept','OnError','OnClientError','OnClientDisconnect','OnClientReceive'],
    [@OnAccept,@OnError,@OnClientError,@OnClientDisconnect,@OnClientReceive]);
end;

procedure TKOLServerSocket.SetOnAccept(const Value: TOnAccept);
begin
  FOnAccept := Value;
  change;
end;

procedure TKOLServerSocket.SetOnClientDisconnect(
  const Value: TOnDisconnect);
begin
  FOnClientDisconnect := Value;
  change;
end;

procedure TKOLServerSocket.SetOnClientError(const Value: TOnError);
begin
  FOnClientError := Value;
  change;
end;

procedure TKOLServerSocket.SetOnClientReceive(const Value: TOnReceive);
begin
  FOnClientReceive := Value;
  change;
end;

procedure TKOLServerSocket.SetOnError(const Value: TOnError);
begin
  FOnError := Value;
  change;
end;

procedure TKOLServerSocket.SetPort(const Value: smallint);
begin
  FPort := Value;
  change;
end;

procedure TKOLServerSocket.SetupFirst(SL: TStringList; const AName, AParent, Prefix: String);
begin
  sl.add(prefix+aname+':=newserver;');
  sl.add(prefix+aname+'.port:='+inttostr(fport)+';');
end;

procedure TKOLServerSocket.SetupLast(SL: TStringList; const AName, AParent, Prefix: String);
begin
//
end;

end.

